/*
 * Click nbfs://nbhost/SystemFileSystem/Templates/Licenses/license-default.txt to change this license
 * Click nbfs://nbhost/SystemFileSystem/Templates/Classes/Main.java to edit this template
 */
package mv3302.run;

import mv3302.CustomerArrivalProcess;
import mv3302.ServerWithReneges;
import simkit.Schedule;
import simkit.random.RandomVariate;
import simkit.random.RandomVariateFactory;
import simkit.stat.CollectionSizeTimeVaryingStats;
import simkit.stat.SimpleStatsTally;
import simkit.stat.SimpleStatsTimeVarying;
import simkit.util.SimplePropertyDumper;

/**
 * Simple test of ServerWithReneges calculating for Number of Arrivals, Number
 * Served, Number Reneges, Percent Reneges Average number in queue, Average
 * Utilization, Avg Delay in Queue Served, Avg Delay in Queue Reneged, and Avg
 * Time in System
 */
public class RunServerWithReneges {

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
//        Instantiate CustomerArrivalProcess
        RandomVariate interarrivalTimeGenerator = RandomVariateFactory.getInstance("Exponential", 1.5);
        RandomVariate serviceTimeGenerator = RandomVariateFactory.getInstance("Gamma", 2.5, 1.2);
        RandomVariate renegeTimeGenerator = RandomVariateFactory.getInstance("Uniform", 4.0, 6.0);
        CustomerArrivalProcess renegingCustomerArrivalProcess = new CustomerArrivalProcess(renegeTimeGenerator, interarrivalTimeGenerator, serviceTimeGenerator);
        System.out.println(renegingCustomerArrivalProcess);
//        Instantiate ServerWithReneges
        int totalNumberServers = 2;

        ServerWithReneges serverWithReneges = new ServerWithReneges(totalNumberServers, serviceTimeGenerator);
        System.out.println(serverWithReneges);

//        The ServerWithReneges will "hear" Arrival(Customer) events
        renegingCustomerArrivalProcess.addSimEventListener(serverWithReneges);

//        This was used for debugging model
        SimplePropertyDumper simplePropertyDumper = new SimplePropertyDumper(true);

//        This listener computes time-varying statictics for Collection states
        CollectionSizeTimeVaryingStats numberInQueueStat = new CollectionSizeTimeVaryingStats("queue");
        serverWithReneges.addPropertyChangeListener(numberInQueueStat);

//        Same as SimpleServer model
        SimpleStatsTimeVarying numberAvailableServersStat = new SimpleStatsTimeVarying("numberAvailableServers");
        serverWithReneges.addPropertyChangeListener(numberAvailableServersStat);

//        Tally statistics are computed in the same way by "listening" 
        SimpleStatsTally timeInSystemStat = new SimpleStatsTally("timeInSystem");
        serverWithReneges.addPropertyChangeListener(timeInSystemStat);

        SimpleStatsTally delayInQueueServedStat = new SimpleStatsTally("delayInQueueServed");
        serverWithReneges.addPropertyChangeListener(delayInQueueServedStat);

        SimpleStatsTally delayInQueueRenegedStat = new SimpleStatsTally("delayInQueueReneged");
        serverWithReneges.addPropertyChangeListener(delayInQueueRenegedStat);

//        Commented out after model debugged
//        renegingCustomerArrivalProcess.addPropertyChangeListener(simplePropertyDumper);
//        serverWithReneges.addPropertyChangeListener(simplePropertyDumper);
//        Was "true" when debugging model
        Schedule.setVerbose(false);
//        Schedule.stopOnEvent(5, "EndService", Entity.class);

//        Run for 100,000 time units
        double stopTime = 100000.0;
        Schedule.stopAtTime(stopTime);

//        Initialized & run 
        Schedule.reset();
        Schedule.startSimulation();

        System.out.printf("%nSimulation ended at time %,.2f%n%n", Schedule.getSimTime());

        System.out.printf("Number Arrivals: %,d%n", renegingCustomerArrivalProcess.getNumberArrivals());
        System.out.printf("Number Served: %,d%n", serverWithReneges.getNumberServed());
        System.out.printf("Number Reneges: %,d%n", renegingCustomerArrivalProcess.getNumberArrivals() - serverWithReneges.getNumberServed());
        double percentReneges = (double) (renegingCustomerArrivalProcess.getNumberArrivals() - serverWithReneges.getNumberServed()) / renegingCustomerArrivalProcess.getNumberArrivals() * 100.00;
        System.out.printf("Percent Reneges: %,.2f%%%n", percentReneges);

        System.out.printf("Avg # in Queue: %,.3f%n", numberInQueueStat.getMean());
        System.out.printf("Avg Utilization: %,.3f%n%n", numberAvailableServersStat.getMean());

//        From directly computing delay in queue, delay in reneged and time in system
        System.out.printf("Avg Delay in Queue Served: %,.3f%n", delayInQueueServedStat.getMean());
        System.out.printf("Avg Delay in Queue Reneged: %,.3f%n", delayInQueueRenegedStat.getMean());
        System.out.printf("Avg Time in System (Served): %,.3f%n", timeInSystemStat.getMean());
    }

}
